@@ -1,6 +1,8 @@
 package net.jpountz.lz4;
 
 /*
+ * Copyright 2020 Charles Allen and the lz4-java contributors.
+ *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
@@ -29,6 +31,7 @@ import java.io.EOFException;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileOutputStream;
+import java.io.SequenceInputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
@@ -274,14 +277,45 @@ public class LZ4FrameIOStreamTest {
     }
   }
 
+  @Test
+  public void testSkippableOnly() throws IOException {
+    final File lz4File = Files.createTempFile("lz4test", ".lz4").toFile();
+    try {
+      try (FileOutputStream fos = new FileOutputStream(lz4File)) {
+        final int skipSize = 1 << 10;
+        final ByteBuffer skipBuffer = ByteBuffer.allocate(skipSize + 8).order(ByteOrder.LITTLE_ENDIAN);
+        skipBuffer.putInt(LZ4FrameInputStream.MAGIC_SKIPPABLE_BASE | 0x00000007); // anything 00 through FF should work
+        skipBuffer.putInt(skipSize);
+        final byte[] skipRandom = new byte[skipSize];
+        new Random(478278L).nextBytes(skipRandom);
+        skipBuffer.put(skipRandom);
+        fos.write(skipBuffer.array());
+      }
+      try (InputStream is = new LZ4FrameInputStream(new FileInputStream(lz4File))) {
+	Assert.assertEquals(-1, is.read());
+      }
+      // Extra one byte at the tail
+      try (InputStream is = new LZ4FrameInputStream(new SequenceInputStream(new FileInputStream(lz4File), new ByteArrayInputStream(new byte[1])))) {
+        is.read();
+        Assert.assertFalse(true);
+      } catch (IOException ex) {
+        // OK
+      } catch (Exception ex) {
+        Assert.assertFalse(true);
+      }
+    } finally {
+      lz4File.delete();
+    }
+  }
 
   @Test
   public void testStreamWithContentSize() throws IOException {
     final File lz4File = Files.createTempFile("lz4test", ".lz4").toFile();
     try {
+      final long knownSize = tmpFile.length();
       try (OutputStream os = new LZ4FrameOutputStream(new FileOutputStream(lz4File),
                                                       LZ4FrameOutputStream.BLOCKSIZE.SIZE_4MB,
-                                                      tmpFile.length(),
+                                                      knownSize,
                                                       LZ4FrameOutputStream.FLG.Bits.BLOCK_INDEPENDENCE,
                                                       LZ4FrameOutputStream.FLG.Bits.CONTENT_CHECKSUM,
                                                       LZ4FrameOutputStream.FLG.Bits.CONTENT_SIZE)) {
@@ -289,6 +323,50 @@ public class LZ4FrameIOStreamTest {
           copy(is, os);
         }
       }
+      try (LZ4FrameInputStream is = new LZ4FrameInputStream(new FileInputStream(lz4File), true)) {
+        Assert.assertEquals(knownSize, is.getExpectedContentSize());
+        Assert.assertTrue(is.isExpectedContentSizeDefined());
+        validateStreamEquals(is, tmpFile);
+      }
+    } finally {
+      lz4File.delete();
+    }
+  }
+
+  @Test
+  public void testStreamWithoutContentSize() throws IOException {
+    final File lz4File = Files.createTempFile("lz4test", ".lz4").toFile();
+    try {
+      try (OutputStream os = new LZ4FrameOutputStream(new FileOutputStream(lz4File),
+              LZ4FrameOutputStream.BLOCKSIZE.SIZE_4MB,
+              LZ4FrameOutputStream.FLG.Bits.BLOCK_INDEPENDENCE,
+              LZ4FrameOutputStream.FLG.Bits.CONTENT_CHECKSUM)) {
+        try (InputStream is = new FileInputStream(tmpFile)) {
+          copy(is, os);
+        }
+      }
+      try (LZ4FrameInputStream is = new LZ4FrameInputStream(new FileInputStream(lz4File), true)) {
+        Assert.assertEquals(-1L, is.getExpectedContentSize());
+        Assert.assertFalse(is.isExpectedContentSizeDefined());
+        validateStreamEquals(is, tmpFile);
+      }
+    } finally {
+      lz4File.delete();
+    }
+  }
+
+  @Test
+  public void testInputOutputWithBlockChecksum() throws IOException {
+    final File lz4File = Files.createTempFile("lz4test", ".lz4").toFile();
+    try {
+      try (OutputStream os = new LZ4FrameOutputStream(new FileOutputStream(lz4File),
+                                                      LZ4FrameOutputStream.BLOCKSIZE.SIZE_64KB,
+                                                      LZ4FrameOutputStream.FLG.Bits.BLOCK_INDEPENDENCE,
+                                                      LZ4FrameOutputStream.FLG.Bits.BLOCK_CHECKSUM)) {
+        try (InputStream is = new FileInputStream(tmpFile)) {
+          copy(is, os);
+        }
+      }
       try (InputStream is = new LZ4FrameInputStream(new FileInputStream(lz4File))) {
         validateStreamEquals(is, tmpFile);
       }
@@ -297,7 +375,6 @@ public class LZ4FrameIOStreamTest {
     }
   }
 
-
   @Test
   public void testInputOutputMultipleFrames() throws IOException {
     final File lz4File = Files.createTempFile("lz4test", ".lz4").toFile();
@@ -321,12 +398,28 @@ public class LZ4FrameIOStreamTest {
           }
         }
       }
-      try (InputStream is = new LZ4FrameInputStream(new FileInputStream(lz4File))) {
+      try (LZ4FrameInputStream is = new LZ4FrameInputStream(new FileInputStream(lz4File))) {
+        try {
+          is.getExpectedContentSize();
+          Assert.assertFalse(true);
+        } catch (UnsupportedOperationException e) {
+          // OK
+        }
+        Assert.assertFalse(is.isExpectedContentSizeDefined());
         validateStreamEquals(is, tmpFile);
         validateStreamEquals(is, tmpFile);
         validateStreamEquals(is, tmpFile);
         validateStreamEquals(is, tmpFile);
       }
+      try (LZ4FrameInputStream is = new LZ4FrameInputStream(new FileInputStream(lz4File), true)) {
+        Assert.assertEquals(-1L, is.getExpectedContentSize());
+        Assert.assertFalse(is.isExpectedContentSizeDefined());
+        validateStreamEquals(is, tmpFile);
+        Assert.assertEquals(-1, is.read());
+        final byte[] tmpBuff = new byte[10];
+        Assert.assertEquals(-1, is.read(tmpBuff, 0, 10));
+        Assert.assertEquals(0, is.skip(1));
+      }
     } finally {
       lz4File.delete();
     }
@@ -427,4 +520,49 @@ public class LZ4FrameIOStreamTest {
       unCompressedFile.delete();
     }
   }
+
+  @Test
+  public void testEmptyLZ4Input() {
+    try (InputStream is = new LZ4FrameInputStream(new ByteArrayInputStream(new byte[0]))) {
+      is.read();
+      Assert.assertFalse(true);
+    } catch (IOException ex) {
+      // OK
+    } catch (Exception ex) {
+      Assert.assertFalse(true);
+    }
+  }
+
+  @Test
+  public void testPrematureMagicNb() throws IOException {
+    try (InputStream is = new LZ4FrameInputStream(new ByteArrayInputStream(new byte[1]))) {
+      is.read();
+      Assert.assertFalse(true);
+    } catch (IOException ex) {
+      // OK
+    } catch (Exception ex) {
+      Assert.assertFalse(true);
+    }
+
+    final File lz4File = Files.createTempFile("lz4test", ".lz4").toFile();
+    try {
+      try (OutputStream os = new LZ4FrameOutputStream(new FileOutputStream(lz4File))) {
+        try (InputStream is = new FileInputStream(tmpFile)) {
+          copy(is, os);
+        }
+      }
+      // Extra one byte at the tail
+      try (InputStream is = new LZ4FrameInputStream(new SequenceInputStream(new FileInputStream(lz4File), new ByteArrayInputStream(new byte[1])))) {
+        validateStreamEquals(is, tmpFile);
+        is.read();
+        Assert.assertFalse(true);
+      } catch (IOException ex) {
+        // OK
+      } catch (Exception ex) {
+        Assert.assertFalse(true);
+      }
+    } finally {
+      lz4File.delete();
+    }
+  }
 }
