@@ -1,6 +1,8 @@
 package net.jpountz.lz4;
 
 /*
+ * Copyright 2020 Adrien Grand and the lz4-java contributors.
+ *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
@@ -23,14 +25,58 @@ import java.io.IOException;
 import java.nio.ByteBuffer;
 import java.nio.ReadOnlyBufferException;
 import java.util.Arrays;
+import java.io.File;
+import java.io.FilenameFilter;
 
 import org.junit.Test;
 import static org.junit.Assert.*;
 
 import com.carrotsearch.randomizedtesting.annotations.Repeat;
+import com.carrotsearch.randomizedtesting.annotations.Seed;
+import com.carrotsearch.randomizedtesting.annotations.Seeds;
 
 public class LZ4Test extends AbstractLZ4Test {
 
+  @Test
+  public void testLockFileOfTemporaryNativeLibrary() {
+    // Load the native library
+    LZ4JNI.LZ4_compressBound(100);
+    String tempFolder = new File(System.getProperty("java.io.tmpdir")).getAbsolutePath();
+    File tempDir = new File(new File(System.getProperty("java.io.tmpdir")).getAbsolutePath());
+    if (!System.getProperty("os.name").contains("Windows")) {
+	// A temporary library must be accompanied by a lock file.
+	// On Windows, JVM does not remove a temporary library on exit.
+	// This means on Windows, there might be a temporary library
+	// that is not accompanied by a lock file when there was
+	// a Java process using lz4-java that was running concurrently
+	// to this test process.
+	File[] tempLibFiles = tempDir.listFiles(new FilenameFilter() {
+		public boolean accept(File dir, String name) {
+		    return name.startsWith("liblz4-java-") && !name.endsWith(".lck");
+		}
+	    });
+	if (tempLibFiles != null) {
+	    for (File tempLibFile : tempLibFiles) {
+		File lckFile = new File(tempLibFile.getAbsolutePath() + ".lck");
+		assertTrue(tempLibFile.getAbsolutePath(), lckFile.exists());
+	    }
+	}
+    }
+    // A lock file must be accompanied by a temporary library.
+    File[] tempLockFiles = tempDir.listFiles(new FilenameFilter() {
+	public boolean accept(File dir, String name) {
+	  return name.startsWith("liblz4-java-") && name.endsWith(".lck");
+	}
+      });
+    if (tempLockFiles != null) {
+      for (File tempLockFile : tempLockFiles) {
+	String tempLockFilePath = tempLockFile.getAbsolutePath();
+	File libFile = new File(tempLockFilePath.substring(0, tempLockFilePath.length() - 4));
+	assertTrue(tempLockFilePath, libFile.exists());
+      }
+    }
+  }
+
   @Test
   @Repeat(iterations=50)
   public void testMaxCompressedLength() {
@@ -46,12 +92,14 @@ public class LZ4Test extends AbstractLZ4Test {
     if (len >= LZ4Constants.RUN_MASK) {
       baos.write(LZ4Constants.RUN_MASK << LZ4Constants.ML_BITS);
       len -= LZ4Constants.RUN_MASK;
+      while (len >= 255) {
+        baos.write(255);
+        len -= 255;
+      }
+      baos.write(len);
+    } else {
+      baos.write(len << LZ4Constants.ML_BITS);
     }
-    while (len >= 255) {
-      baos.write(255);
-      len -= 255;
-    }
-    baos.write(len);
     try {
       baos.write(decompressed);
     } catch (IOException e) {
@@ -94,6 +142,10 @@ public class LZ4Test extends AbstractLZ4Test {
     assertArrayEquals(decompressed, restored);
   }
 
+  @Seeds({
+    @Seed("69CCC65267F3C588"),  // Seed that generates len < 16
+    @Seed()
+  })
   @Test
   public void testUncompressSafeWorstCase() {
     for (LZ4SafeDecompressor decompressor : SAFE_DECOMPRESSORS) {
@@ -108,6 +160,11 @@ public class LZ4Test extends AbstractLZ4Test {
     for (Tester<?> tester : Arrays.asList(Tester.BYTE_ARRAY, Tester.BYTE_BUFFER, Tester.BYTE_ARRAY_WITH_LENGTH, Tester.BYTE_BUFFER_WITH_LENGTH)) {
       testRoundTrip(tester, data, off, len, compressor, decompressor, decompressor2);
     }
+    if (data.length == len && off == 0) {
+      for (SrcDestTester<?> tester : Arrays.asList(SrcDestTester.BYTE_ARRAY, SrcDestTester.BYTE_BUFFER, SrcDestTester.BYTE_ARRAY_WITH_LENGTH, SrcDestTester.BYTE_BUFFER_WITH_LENGTH)) {
+        testRoundTrip(tester, data, compressor, decompressor, decompressor2);
+      }
+    }
   }
 
   public <T> void testRoundTrip(
@@ -146,6 +203,9 @@ public class LZ4Test extends AbstractLZ4Test {
     }
 
     if (tester != Tester.BYTE_ARRAY_WITH_LENGTH && tester != Tester.BYTE_BUFFER_WITH_LENGTH) {
+      // LZ4DecompressorWithLength will succeed in decompression
+      // because it ignores destLen.
+
       if (len > 0) {
 	// decompression dest is too small
 	try {
@@ -164,34 +224,81 @@ public class LZ4Test extends AbstractLZ4Test {
       } catch (LZ4Exception e) {
 	// OK
       }
+    }
 
-      // try decompression when only the size of the compressed buffer is known
-      if (len > 0) {
-	tester.fill(restored, randomByte());
-	assertEquals(len, tester.decompress(decompressor2, compressed, 0, compressedLen, restored, 0, len));
-	assertArrayEquals(Arrays.copyOfRange(data, off, off + len), tester.copyOf(restored, 0, len));
-	tester.fill(restored, randomByte());
-      } else {
-	assertEquals(0, tester.decompress(decompressor2, compressed, 0, compressedLen, tester.allocate(1), 0, 1));
-      }
+    // try decompression when only the size of the compressed buffer is known
+    if (len > 0) {
+      tester.fill(restored, randomByte());
+      assertEquals(len, tester.decompress(decompressor2, compressed, 0, compressedLen, restored, 0, len));
+      assertArrayEquals(Arrays.copyOfRange(data, off, off + len), tester.copyOf(restored, 0, len));
+      tester.fill(restored, randomByte());
+    } else {
+      assertEquals(0, tester.decompress(decompressor2, compressed, 0, compressedLen, tester.allocate(1), 0, 1));
+    }
 
-      // over-estimated compressed length
-      try {
-	final int decompressedLen = tester.decompress(decompressor2, compressed, 0, compressedLen + 1, tester.allocate(len + 100), 0, len + 100);
-	fail("decompressedLen=" + decompressedLen);
-      } catch (LZ4Exception e) {
-	// OK
-      }
+    // over-estimated compressed length
+    try {
+      final int decompressedLen = tester.decompress(decompressor2, compressed, 0, compressedLen + 1, tester.allocate(len + 100), 0, len + 100);
+      fail("decompressedLen=" + decompressedLen);
+    } catch (LZ4Exception e) {
+      // OK
+    }
 
-      // under-estimated compressed length
-      try {
-	final int decompressedLen = tester.decompress(decompressor2, compressed, 0, compressedLen - 1, tester.allocate(len + 100), 0, len + 100);
-	if (!(decompressor2 instanceof LZ4JNISafeDecompressor)) {
-	  fail("decompressedLen=" + decompressedLen);
-	}
-      } catch (LZ4Exception e) {
-	// OK
+    // under-estimated compressed length
+    try {
+      final int decompressedLen = tester.decompress(decompressor2, compressed, 0, compressedLen - 1, tester.allocate(len + 100), 0, len + 100);
+      if (!(decompressor2 instanceof LZ4JNISafeDecompressor)) {
+        fail("decompressedLen=" + decompressedLen);
       }
+    } catch (LZ4Exception e) {
+      // OK
+    }
+  }
+
+  public <T> void testRoundTrip(SrcDestTester<T> tester,
+                                byte[] data,
+                                LZ4Compressor compressor,
+                                LZ4FastDecompressor decompressor,
+                                LZ4SafeDecompressor decompressor2) {
+    final T original = tester.copyOf(data);
+    final int maxCompressedLength = tester.maxCompressedLength(data.length);
+    final T compressed = tester.allocate(maxCompressedLength);
+    final int compressedLen = tester.compress(compressor,
+                                              original,
+                                              compressed);
+    if (original instanceof ByteBuffer) {
+      assertEquals(data.length, ((ByteBuffer)original).position());
+      assertEquals(compressedLen, ((ByteBuffer)compressed).position());
+      ((ByteBuffer)original).rewind();
+      ((ByteBuffer)compressed).rewind();
+    }
+
+    // test decompression
+    final T restored = tester.allocate(data.length);
+    assertEquals(compressedLen, tester.decompress(decompressor, compressed, restored));
+    if (original instanceof ByteBuffer) {
+      assertEquals(compressedLen, ((ByteBuffer)compressed).position());
+      assertEquals(data.length, ((ByteBuffer)restored).position());
+    }
+    assertArrayEquals(data, tester.copyOf(restored, 0, data.length));
+    if (original instanceof ByteBuffer) {
+      ((ByteBuffer)compressed).rewind();
+      ((ByteBuffer)restored).rewind();
+    }
+
+    // try decompression when only the size of the compressed buffer is known
+    tester.fill(restored, randomByte());
+    // Truncate the compressed buffer to the exact size
+    final T compressedExactSize = tester.copyOf(tester.copyOf(compressed, 0, compressedLen));
+    assertEquals(data.length, tester.decompress(decompressor2, compressedExactSize, restored));
+    if (original instanceof ByteBuffer) {
+      assertEquals(compressedLen, ((ByteBuffer)compressedExactSize).position());
+      assertEquals(data.length, ((ByteBuffer)restored).position());
+    }
+    assertArrayEquals(data, tester.copyOf(restored, 0, data.length));
+    if (original instanceof ByteBuffer) {
+      ((ByteBuffer)compressedExactSize).rewind();
+      ((ByteBuffer)restored).rewind();
     }
   }
 
